@@ -5,18 +5,24 @@ import os.path
 import sys
 import runpy
 import tempfile
+import subprocess
 
 
 __all__ = ["version", "bootstrap"]
 
 _WHEEL_DIR = "/usr/share/python-wheels/"
 
+_wheels = {}
+
 def _get_most_recent_wheel_version(pkg):
     prefix = os.path.join(_WHEEL_DIR, "{}-".format(pkg))
-    suffix = "-py2.py3-none-any.whl"
-    pattern = "{}*{}".format(prefix, suffix)
-    versions = (p[len(prefix):-len(suffix)] for p in glob.glob(pattern))
-    return str(max(versions, key=distutils.version.LooseVersion))
+    _wheels[pkg] = {}
+    for suffix in "-py2.py3-none-any.whl", "-py3-none-any.whl":
+        pattern = "{}*{}".format(prefix, suffix)
+        for path in glob.glob(pattern):
+            version_str = path[len(prefix):-len(suffix)]
+            _wheels[pkg][version_str] = os.path.basename(path)
+    return str(max(_wheels[pkg], key=distutils.version.LooseVersion))
 
 
 _SETUPTOOLS_VERSION = _get_most_recent_wheel_version("setuptools")
@@ -30,22 +36,18 @@ _PROJECTS = [
 
 
 def _run_pip(args, additional_paths=None):
-    # Add our bundled software to the sys.path so we can import it
-    if additional_paths is not None:
-        sys.path = additional_paths + sys.path
-
-    # Invoke pip as if it's the main module, and catch the exit.
-    backup_argv = sys.argv[:]
-    sys.argv[1:] = args
-    try:
-        # run_module() alters sys.modules and sys.argv, but restores them at exit
-        runpy.run_module("pip", run_name="__main__", alter_sys=True)
-    except SystemExit as exc:
-        return exc.code
-    finally:
-        sys.argv[:] = backup_argv
-
-    raise SystemError("pip did not exit, this should never happen")
+    # Run the bootstraping in a subprocess to avoid leaking any state that happens
+    # after pip has executed. Particulary, this avoids the case when pip holds onto
+    # the files in *additional_paths*, preventing us to remove them at the end of the
+    # invocation.
+    code = f"""
+import runpy
+import sys
+sys.path = {additional_paths or []} + sys.path
+sys.argv[1:] = {args}
+runpy.run_module("pip", run_name="__main__", alter_sys=True)
+"""
+    return subprocess.run([sys.executable, "-c", code], check=True).returncode
 
 
 def version():
@@ -93,6 +95,8 @@ def _bootstrap(*, root=None, upgrade=False, user=False,
     if altinstall and default_pip:
         raise ValueError("Cannot use altinstall and default_pip together")
 
+    sys.audit("ensurepip.bootstrap", root)
+
     _disable_pip_configuration_settings()
 
     # By default, installing pip and setuptools installs all of the
@@ -113,12 +117,11 @@ def _bootstrap(*, root=None, upgrade=False, user=False,
         # additional paths that need added to sys.path
         additional_paths = []
         for project, version, py_tag in _PROJECTS:
-            wheel_name = "{}-{}-{}-none-any.whl".format(project, version, py_tag)
+            wheel_name = _wheels[project][version]
             with open(os.path.join(_WHEEL_DIR, wheel_name), "rb") as sfp:
                 with open(os.path.join(tmpdir, wheel_name), "wb") as fp:
                     fp.write(sfp.read())
 
-
             additional_paths.append(os.path.join(tmpdir, wheel_name))
 
         # Construct the arguments to be passed to the pip command
